@@ -45,6 +45,9 @@ static void aisleriot_slot_renderer_set_cache (AisleriotSlotRenderer *srend,
 
 static void completed_cb (AisleriotSlotRenderer *srend);
 
+static const ClutterColor white = { 0xff, 0xff, 0xff, 0xff };
+static const ClutterColor default_highlight_color = { 0, 0, 0xaa, 0xff };
+
 G_DEFINE_TYPE (AisleriotSlotRenderer, aisleriot_slot_renderer,
                CLUTTER_TYPE_ACTOR);
 
@@ -60,6 +63,7 @@ struct _AisleriotSlotRendererPrivate
 
   Slot *slot;
 
+  ClutterColor highlight_color;
   gboolean show_highlight;
   gint highlight_start;
 
@@ -85,6 +89,7 @@ enum
   PROP_CACHE,
   PROP_SLOT,
   PROP_HIGHLIGHT,
+  PROP_HIGHLIGHT_COLOR,
   PROP_REVEALED_CARD,
   PROP_ANIMATION_LAYER
 };
@@ -122,7 +127,7 @@ aisleriot_slot_renderer_class_init (AisleriotSlotRendererClass *klass)
   g_object_class_install_property (gobject_class, PROP_SLOT, pspec);
 
   pspec = g_param_spec_int ("highlight", NULL, NULL,
-                            -1, G_MAXINT, -1,
+                            -1, G_MAXINT, G_MAXINT,
                             G_PARAM_WRITABLE |
                             G_PARAM_READABLE |
                             G_PARAM_STATIC_NAME |
@@ -130,8 +135,27 @@ aisleriot_slot_renderer_class_init (AisleriotSlotRendererClass *klass)
                             G_PARAM_STATIC_BLURB);
   g_object_class_install_property (gobject_class, PROP_HIGHLIGHT, pspec);
 
+
+#if 0
+  pspec = clutter_param_spec_color ("highlight-color", NULL, NULL,
+                                    &default_highlight_color,
+                                    G_PARAM_WRITABLE |
+                                    G_PARAM_STATIC_NAME |
+                                    G_PARAM_STATIC_NICK |
+                                    G_PARAM_STATIC_BLURB |
+                                    G_PARAM_CONSTRUCT);
+#else
+  pspec = g_param_spec_boxed ("highlight-color", NULL, NULL,
+                              CLUTTER_TYPE_COLOR,
+                              G_PARAM_WRITABLE |
+                              G_PARAM_STATIC_NAME |
+                              G_PARAM_STATIC_NICK |
+                              G_PARAM_STATIC_BLURB);
+#endif
+  g_object_class_install_property (gobject_class, PROP_HIGHLIGHT_COLOR, pspec);
+
   pspec = g_param_spec_int ("revealed-card", NULL, NULL,
-                            -1, G_MAXINT, 0,
+                            -1, G_MAXINT, -1,
                             G_PARAM_WRITABLE |
                             G_PARAM_READABLE |
                             G_PARAM_STATIC_NAME |
@@ -158,13 +182,14 @@ aisleriot_slot_renderer_init (AisleriotSlotRenderer *self)
 
   priv = self->priv = AISLERIOT_SLOT_RENDERER_GET_PRIVATE (self);
 
+  priv->revealed_card = -1;
   priv->highlight_start = G_MAXINT;
+  priv->highlight_color = default_highlight_color;
+
   priv->animations = g_array_new (FALSE, FALSE, sizeof (AnimationData));
   priv->timeline = clutter_timeline_new_for_duration (500);
   g_signal_connect_swapped (priv->timeline, "completed",
                             G_CALLBACK (completed_cb), self);
-
-  priv->revealed_card = -1;
 }
 
 static void
@@ -216,6 +241,9 @@ aisleriot_slot_renderer_set_cache (AisleriotSlotRenderer *srend,
 {
   AisleriotSlotRendererPrivate *priv = srend->priv;
 
+  if (cache == priv->cache)
+    return;
+
   if (cache)
     g_object_ref (cache);
 
@@ -248,6 +276,11 @@ aisleriot_slot_renderer_set_property (GObject *object,
                                              g_value_get_int (value));
       break;
 
+    case PROP_HIGHLIGHT_COLOR:
+      aisleriot_slot_renderer_set_highlight_color (srend,
+                                                   g_value_get_boxed (value));
+      break;
+
     case PROP_REVEALED_CARD:
       aisleriot_slot_renderer_set_revealed_card (srend,
                                                  g_value_get_int (value));
@@ -288,6 +321,7 @@ aisleriot_slot_renderer_get_property (GObject *object,
                           aisleriot_slot_renderer_get_animation_layer (srend));
       break;
 
+    case PROP_HIGHLIGHT_COLOR:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -300,17 +334,13 @@ aisleriot_slot_renderer_paint_card (AisleriotSlotRenderer *srend,
 {
   AisleriotSlotRendererPrivate *priv = srend->priv;
   Card card = CARD (priv->slot->cards->data[card_num]);
-  gboolean is_highlighted;
   CoglHandle cogl_tex;
   guint tex_width, tex_height;
   int cardx, cardy;
-  static const ClutterColor white = { 0xff, 0xff, 0xff, 0xff };
 
-  is_highlighted = priv->show_highlight && (card_num >= priv->highlight_start);
-
-  cogl_tex = games_card_textures_cache_get_card_texture (priv->cache,
-                                                    card,
-                                                    is_highlighted);
+  cogl_tex = games_card_textures_cache_get_card_texture (priv->cache, card);
+  if (G_UNLIKELY (cogl_tex == COGL_INVALID_HANDLE))
+    return;
 
   tex_width = cogl_texture_get_width (cogl_tex);
   tex_height = cogl_texture_get_height (cogl_tex);
@@ -326,6 +356,19 @@ aisleriot_slot_renderer_paint_card (AisleriotSlotRenderer *srend,
                           CLUTTER_INT_TO_FIXED (cardx + tex_width),
                           CLUTTER_INT_TO_FIXED (cardy + tex_height),
                           0, 0, CFX_ONE, CFX_ONE);
+
+  if (priv->show_highlight && (card_num >= priv->highlight_start)) {
+    ClutterColor color = priv->highlight_color;
+
+    color.alpha = 0x80;
+    cogl_color (&color);
+    cogl_texture_rectangle (cogl_tex,
+                            CLUTTER_INT_TO_FIXED (cardx),
+                            CLUTTER_INT_TO_FIXED (cardy),
+                            CLUTTER_INT_TO_FIXED (cardx + tex_width),
+                            CLUTTER_INT_TO_FIXED (cardy + tex_height),
+                            0, 0, CFX_ONE, CFX_ONE);
+  }
 }
 
 static void
@@ -349,10 +392,11 @@ aisleriot_slot_renderer_paint (ClutterActor *actor)
   if (priv->slot->cards->len <= priv->animations->len) {
     CoglHandle cogl_tex;
     guint tex_width, tex_height;
-    static const ClutterColor white = { 0xff, 0xff, 0xff, 0xff };
 
-    cogl_tex = games_card_textures_cache_get_slot_texture (priv->cache,
-                                                      priv->show_highlight);
+    cogl_tex = games_card_textures_cache_get_slot_texture (priv->cache);
+    if (G_UNLIKELY (cogl_tex == COGL_INVALID_HANDLE))
+      goto paint_cards;
+
     tex_width = cogl_texture_get_width (cogl_tex);
     tex_height = cogl_texture_get_height (cogl_tex);
 
@@ -362,8 +406,19 @@ aisleriot_slot_renderer_paint (ClutterActor *actor)
                             CLUTTER_INT_TO_FIXED (tex_width),
                             CLUTTER_INT_TO_FIXED (tex_height),
                             0, 0, CFX_ONE, CFX_ONE);
+
+    if (priv->show_highlight) {
+      cogl_color (&priv->highlight_color);
+      cogl_texture_rectangle (cogl_tex,
+                              0, 0,
+                              CLUTTER_INT_TO_FIXED (tex_width),
+                              CLUTTER_INT_TO_FIXED (tex_height),
+                              0, 0, CFX_ONE, CFX_ONE);
+    }
   }
 
+paint_cards:
+
   if (n_cards > priv->animations->len) {
     guint first_card, last_card;
 
@@ -376,7 +431,8 @@ aisleriot_slot_renderer_paint (ClutterActor *actor)
         aisleriot_slot_renderer_paint_card (srend, i);
 
     /* Paint the revealed card after all of the other cards so that it
-       will appeear on top */
+     * will appeear on top.
+     */
     if (priv->revealed_card >= first_card && priv->revealed_card < last_card)
       aisleriot_slot_renderer_paint_card (srend, priv->revealed_card);
   }
@@ -408,6 +464,22 @@ aisleriot_slot_renderer_set_highlight (AisleriotSlotRenderer *srend,
   g_object_notify (G_OBJECT (srend), "highlight");
 }
 
+void
+aisleriot_slot_renderer_set_highlight_color (AisleriotSlotRenderer *srend,
+                                             const ClutterColor *color)
+{
+  AisleriotSlotRendererPrivate *priv = srend->priv;
+
+  g_return_if_fail (color != NULL);
+
+  if (clutter_color_equal (color, &priv->highlight_color))
+    return;
+
+  priv->highlight_color = *color;
+
+  g_object_notify (G_OBJECT (srend), "highlight-color");
+}
+
 gint
 aisleriot_slot_renderer_get_revealed_card (AisleriotSlotRenderer *srend)
 {
@@ -507,21 +579,16 @@ aisleriot_slot_renderer_set_animations (AisleriotSlotRenderer *srend,
     ClutterAlpha *alpha;
     ClutterKnot knots[2];
     Card card = CARD (priv->slot->cards->data[card_num]);
-    guint card_width, card_height;
-    CoglHandle cogl_tex;
+    guint card_width = 0, card_height = 0;
 
     memset (&anim_data, 0, sizeof (anim_data));
 
-    anim_data.card_tex = aisleriot_card_new (priv->cache, card);
+    anim_data.card_tex = aisleriot_card_new (priv->cache, card, &priv->highlight_color);
     g_object_ref_sink (anim_data.card_tex);
     if (priv->animation_layer)
       clutter_container_add (priv->animation_layer,
                              CLUTTER_ACTOR (anim_data.card_tex), NULL);
 
-    cogl_tex = games_card_textures_cache_get_card_texture (priv->cache, card, FALSE);
-    card_width = cogl_texture_get_width (cogl_tex);
-    card_height = cogl_texture_get_height (cogl_tex);
-
     clutter_actor_set_position (anim_data.card_tex,
                                 anims[i].cardx, anims[i].cardy);
 
@@ -541,6 +608,15 @@ aisleriot_slot_renderer_set_animations (AisleriotSlotRenderer *srend,
     clutter_behaviour_apply (anim_data.move, anim_data.card_tex);
 
     if (anims[i].face_down != card.attr.face_down) {
+      CoglHandle cogl_tex;
+
+      cogl_tex = games_card_textures_cache_get_card_texture (priv->cache, card);
+      if (G_UNLIKELY (cogl_tex == COGL_INVALID_HANDLE))
+        continue; // FIXMEchpe this doesn't look right...
+
+      card_width = cogl_texture_get_width (cogl_tex);
+      card_height = cogl_texture_get_height (cogl_tex);
+
       gint center_x = card_width / 2;
       gint center_y = card_height / 2;
 
