@@ -22,7 +22,6 @@
 #include <config.h>
 
 #include <string.h>
-#include <locale.h>
 
 #include <libguile.h>
 
@@ -56,10 +55,12 @@
 #define HELP_EXT "xhtml"
 #endif /* G_OS_WIN32 */
 
+#include <libgames-support/games-debug.h>
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-stock.h>
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-sound.h>
+#include <libgames-support/games-string-utils.h>
 
 #ifdef WITH_SMCLIENT
 #include <libgames-support/eggsmclient.h>
@@ -362,7 +363,7 @@ help_hook (GtkWindow *parent,
     if (game_file != NULL) {
       char *game_name;
  
-      game_name = aisleriot_util_get_display_filename (game_file);
+      game_name = games_filename_to_display_name (game_file);
       primary = g_strdup_printf (_("Could not show help for “%s”"), game_name);
       g_free (game_name);
     } else {
@@ -679,22 +680,9 @@ main_prog (void *closure, int argc, char *argv[])
 int
 main (int argc, char *argv[])
 {
-  setlocale (LC_ALL, "");
-
-#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS) || defined(HAVE_GSTREAMER)
-  /* If we're going to use gnome-vfs or gstreamer, we need to init threads before
-   * calling any glib functions.
-   */
-  g_thread_init (NULL);
-#endif
-
   if (!games_runtime_init ("aisleriot"))
     return 1;
 
-  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-
   scm_boot_guile (argc, argv, main_prog, NULL); /* no return */
 
   return 0;
