@@ -156,12 +156,12 @@ bj_get_hand_results (gint dealer_count, gint player_count)
 
         if (dealer_count > 21)
                 dealer_count = 0;
-        if (player_count <= 21)
+        if (player_count <= 21) {
                 if (player_count > dealer_count)
                         results = 1;
-                else 
-                        if (player_count == dealer_count)
-                                results = 0;
+                else if (player_count == dealer_count)
+                        results = 0;
+        }
 
         return results;
 }
@@ -261,7 +261,8 @@ bj_hand_new5 (gpointer data)
                                                                 FALSE);
                 }
                 if (dealer->cards[0].value () == 1) {
-                        if (insurance = get_insurance_choice ())
+                        insurance = get_insurance_choice ();
+                        if (insurance)
                                 bj_adjust_balance (-1 * player->wager / 2);
                 }
         }
@@ -392,13 +393,30 @@ bj_hand_new ()
         bj_hand_new1 (NULL);
 }
 
+static gint
+bj_count_live_hands ()
+{
+	// Live hands compete with the dealer hand.
+	gint live_hands = 0; 
+	player = (PlayerHand*) g_list_nth_data (playerHands,0);
+	while (player != NULL) {
+		if ((player->getCount () < 22 ) 
+		    && !((player->getCards () == 2) && (player->getCount () == 21)))
+			live_hands++;
+		player = player->nextHand;
+	}
+	return live_hands;
+}
+
 static gboolean
 bj_hand_finish1 (gpointer data)
 {
-        if (dealer->getCount () < 17 
-            || (rules->getHitSoft17 () 
-                && dealer->getCount () == 17 
-                && dealer->getSoft ())) {
+        if (!allSettled
+	    && bj_count_live_hands () 
+	    && (dealer->getCount () < 17 
+                || (rules->getHitSoft17 () 
+                    && dealer->getCount () == 17 
+                    && dealer->getSoft ()))) {
                 // Finish dealer hand.
                 bj_deal_card_to_dealer_distribution ();
                 bj_draw_refresh_screen ();
@@ -412,11 +430,12 @@ bj_hand_finish1 (gpointer data)
                                 dealer_count = 0;
                         player = (PlayerHand*) g_list_nth_data (playerHands,0);
                         while (player != NULL) {
-                                if (player->getCount () <= 21)
+                                if (player->getCount () <= 21) {
                                         if (player->getCount () > dealer_count)
                                                 bj_adjust_balance (player->wager * 2);
                                         else if (player->getCount () == dealer_count)
                                                 bj_adjust_balance (player->wager);
+                                }
                                 player = player->nextHand;
                         }
                 }
@@ -640,7 +659,7 @@ bj_hand_finish_play ()
                         else
                                 check_splits = FALSE;
 
-                        if (player->getCount () >= 21) 
+                        if (player->getCount () >= 21) {
                                 if (player->nextHand == NULL) {
                                         check_splits = FALSE;
                                         allSettled = FALSE;
@@ -648,6 +667,7 @@ bj_hand_finish_play ()
                                 }
                                 else
                                         player = player->nextHand;
+                        }
                         
                         bj_hand_show_dealer_probabilities ();
                         bj_hand_show_options ();
