@@ -359,6 +359,10 @@ class Player(game.ChessPlayer):
         isSelf = player is self and self.moving
         self.moving = False
         self.connection.reportMove(move.canMove, isSelf)
+        
+    def onUndoMove(self):
+        """Called by game.ChessPlayer"""
+        self.connection.undoMove()
 
     def readyToMove(self):
         """Called by game.ChessPlayer"""
@@ -455,7 +459,7 @@ class Player(game.ChessPlayer):
                 
                     # Bridge information between the application and the engine
                     os.write(targets[fd], data)
-        except Exception, e:
+        except:
             # Kill the child and exit
             try:
                 os.kill(enginePID, signal.SIGQUIT)
