@@ -85,17 +85,9 @@ static const gint field_sizes[MAX_SIZE][4] = {
 };
 
 static const GamesScoresCategory scorecats[] = {
-  { "Small",  N_("Small") },
-  { "Medium", N_("glines|Medium") },
-  { "Large",  N_("Large") },
-  GAMES_SCORES_LAST_CATEGORY
-};
-
-static const GamesScoresDescription scoredesc = {
-  scorecats,
-  "Small",
-  "glines",
-  GAMES_SCORES_STYLE_PLAIN_DESCENDING
+  { "Small",  NC_("board size", "Small")  },
+  { "Medium", NC_("board size", "Medium") },
+  { "Large",  NC_("board size", "Large")  }
 };
 
 static GamesScores *highscores;
@@ -1446,6 +1438,8 @@ game_props_callback (void)
   GtkWidget *table;
   GtkWidget *vbox;
   GtkWidget *button;
+  int i;
+  GSList *group;
 
   if (!pref_dialog) {
     pref_dialog = gtk_dialog_new_with_buttons (_("Five or More Preferences"),
@@ -1508,33 +1502,21 @@ game_props_callback (void)
     gtk_container_add (GTK_CONTAINER (frame), fv);
     gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
 
-    button = gtk_radio_button_new_with_mnemonic (NULL, _("_Small"));
-    if (game_size == SMALL)
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (button, "clicked",
-                      G_CALLBACK (size_callback), (gpointer) SMALL);
+    group = NULL;
+    for (i = 0; i < G_N_ELEMENTS (scorecats); ++i) {
+      button = gtk_radio_button_new_with_mnemonic (group, g_dpgettext2 (NULL, "board size", scorecats[i].name));
+      if (game_size == i)
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
-    gtk_container_add (GTK_CONTAINER (fv), button);
+      g_signal_connect (button, "clicked",
+                        G_CALLBACK (size_callback), GINT_TO_POINTER (i + 1));
 
-    button = gtk_radio_button_new_with_mnemonic
-      (gtk_radio_button_get_group (GTK_RADIO_BUTTON (button)), _("_Medium"));
+      gtk_box_pack_start (GTK_BOX (fv), button, FALSE, FALSE, 0);
 
-    if (game_size == MEDIUM)
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (button, "clicked",
-                      G_CALLBACK (size_callback), (gpointer) MEDIUM);
-    gtk_container_add (GTK_CONTAINER (fv), button);
+      group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
+    }
 
-    button = gtk_radio_button_new_with_mnemonic
-      (gtk_radio_button_get_group (GTK_RADIO_BUTTON (button)), _("_Large"));
-    if (game_size == LARGE)
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (button, "clicked",
-                      G_CALLBACK (size_callback), (gpointer) LARGE);
-    gtk_container_add (GTK_CONTAINER (fv), button);
-
-
-    frame = games_frame_new (Q_("glines|General"));
+    frame = games_frame_new (C_("preferences", "General"));
     fv = gtk_vbox_new (FALSE, FALSE);
     gtk_box_set_spacing (GTK_BOX (fv), 6);
     gtk_container_add (GTK_CONTAINER (frame), fv);
@@ -1801,13 +1783,6 @@ main (int argc, char *argv[])
   EggSMClient *sm_client;
 #endif /* WITH_SMCLIENT */
 
-#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
-  /* If we're going to use gnome-vfs, we need to init threads before
-   * calling any glib functions.
-   */
-  g_thread_init (NULL);
-#endif
-
   if (!games_runtime_init ("glines"))
     return 1;
 
@@ -1815,10 +1790,6 @@ main (int argc, char *argv[])
 
   rgen = g_rand_new ();
 
-  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
@@ -1841,7 +1812,11 @@ main (int argc, char *argv[])
 
   games_conf_initialise ("GLines");
 
-  highscores = games_scores_new (&scoredesc);
+  highscores = games_scores_new ("glines",
+                                 scorecats, G_N_ELEMENTS (scorecats),
+                                 "board size", NULL,
+                                 0 /* default category */,
+                                 GAMES_SCORES_STYLE_PLAIN_DESCENDING);
 
   init_config ();
 
