@@ -27,7 +27,7 @@
 #include <libgames-support/games-runtime.h>
 
 #ifdef HAVE_CLUTTER
-#include <clutter-gtk/gtk-clutter-embed.h>
+#include <libgames-support/games-clutter-embed.h>
 #endif
 
 #include "tetris.h"
@@ -38,17 +38,11 @@ main(int argc, char *argv[])
 	gboolean retval;
 	GError *error = NULL;
 
-	g_thread_init (NULL);
-
 	if (!games_runtime_init ("gnometris"))
 		return 1;
 
 	setgid_io_init ();
 
-	bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain(GETTEXT_PACKAGE);
-
 	int cmdlineLevel = 0;
 
 	const GOptionEntry options[] =
@@ -81,7 +75,7 @@ main(int argc, char *argv[])
 	games_conf_initialise ("Gnometris");
 
 #ifdef HAVE_CLUTTER
-        if (gtk_clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS) {
+        if (games_clutter_init_with_args (NULL, NULL, NULL, NULL, NULL, &error) != CLUTTER_INIT_SUCCESS) {
 		g_printerr ("Failed to initialise clutter: %s\n", error->message);
 		g_error_free (error);
 		return 1;
