@@ -120,13 +120,8 @@ const static GOptionEntry yahtzee_options[] = {
 };
 
 static const GamesScoresCategory category_array[] = {
-    {"Regular", "Regular"}, {"Colors", "Colors"}, GAMES_SCORES_LAST_CATEGORY };
-
-static const GamesScoresDescription scoredesc = { 
-  category_array,
-  "Regular",
-  "gtali",
-  GAMES_SCORES_STYLE_PLAIN_DESCENDING
+  {"Regular", NC_("game type", "Regular") },
+  {"Colors",  NC_("game type", "Colors")}
 };
 
 GamesScores *highscores;
@@ -876,17 +871,11 @@ main (int argc, char *argv[])
   gboolean retval;
   GError *error = NULL;
 
-  g_thread_init (NULL);
-
   if (!games_runtime_init ("gtali"))
     return 1;
 
   setgid_io_init ();
 
-  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-
   /* Reset all yahtzee variables before parsing args */
   YahtzeeInit ();
 
@@ -947,7 +936,11 @@ main (int argc, char *argv[])
       exit(0);
   }
 
-  highscores = games_scores_new (&scoredesc);
+  highscores = games_scores_new ("gtali",
+                                 category_array, G_N_ELEMENTS (category_array),
+                                 "game type", NULL,
+                                 0 /* default category */,
+                                 GAMES_SCORES_STYLE_PLAIN_DESCENDING);
 
   gtk_window_set_default_icon_name ("gnome-tali");
 
