@@ -87,10 +87,8 @@ static void games_scores_dialog_add_category (GamesScoresDialog *self,
 static void games_scores_dialog_load_categories (GamesScoresCategory *cat, 
 						 GamesScoresDialog *self) 
 {
-    /* Note the Q_ here, this is so we can use context tags on similar
-     * names. (Medium is one that can cause trouble for translators 
-     * in different contexts.) */
-    games_scores_dialog_add_category (self, cat->key, Q_(cat->name));
+    /* category->name is already translated here! */
+    games_scores_dialog_add_category (self, cat->key, cat->name);
 }
 
 /**
@@ -165,9 +163,9 @@ GtkWidget * games_scores_dialog_new (GtkWindow *parent_window, GamesScores *scor
   gtk_window_set_title (GTK_WINDOW (dialog), title);
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent_window));
 
-  games_scores_category_foreach (scores, 
-				 (GamesScoresCategoryForeachFunc) games_scores_dialog_load_categories, 
-				 dialog);
+  _games_scores_category_foreach (scores, 
+                                  (GamesScoresCategoryForeachFunc) games_scores_dialog_load_categories,
+                                  dialog);
 
   if (dialog->_priv->catcounter <= 1) {
     gtk_widget_hide (dialog->_priv->catbar);
@@ -182,12 +180,18 @@ static void games_scores_dialog_name_edited (GtkCellRendererText *cell,
 					     GamesScoresDialog *self)
 {
   GtkTreeIter iter;
+  gchar *old_name = NULL;
 
   gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (self->_priv->list), 
 				       &iter, path);
+
+  /* Get old name for comparison */
+  gtk_tree_model_get (GTK_TREE_MODEL (self->_priv->list),
+                      &iter, 0, &old_name, -1);
+                           
   gtk_list_store_set (self->_priv->list, &iter, 0, new_text, -1);
 
-  games_scores_update_score (self->_priv->scores, new_text);
+  games_scores_update_score_name (self->_priv->scores, new_text, old_name);
 }
 
 /* Prevent editing of any cell in the high score list but the one we set. */
@@ -508,6 +512,7 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
 			FALSE, FALSE, 0);	
  
   self->_priv->combo = gtk_combo_box_new_text ();
+  gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (self->_priv->combo), FALSE);
   gtk_box_pack_start (GTK_BOX (self->_priv->catbar), 
 			self->_priv->combo, TRUE, TRUE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (self->_priv->label), self->_priv->combo);
@@ -551,6 +556,8 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
 
   gtk_window_set_destroy_with_parent (GTK_WINDOW (self), TRUE);
 
+  gtk_widget_grab_focus (self->_priv->combo);
+
   gtk_widget_show_all (vbox);
   gtk_widget_hide (self->_priv->hdiv);
   gtk_widget_hide (self->_priv->message);
