@@ -104,12 +104,6 @@ static guint timeout_counter = HINT_BLINK_NUM + 1;
 
 static GamesScores *highscores;
 
-static const GamesScoresDescription scoredesc = { NULL,
-  "Easy",
-  "mahjongg",
-  GAMES_SCORES_STYLE_TIME_ASCENDING
-};
-
 enum {
   GAME_RUNNING = 0,
   GAME_WAITING,
@@ -579,7 +573,10 @@ fill_map_menu (GtkWidget * menu)
   gint lp;
 
   for (lp = 0; lp < nmaps; lp++) {
-    gtk_combo_box_append_text (GTK_COMBO_BOX (menu), Q_ (maps[lp].name));
+    const char *display_name;
+
+    display_name = g_dpgettext2 (NULL, "mahjongg map name", maps[lp].name);
+    gtk_combo_box_append_text (GTK_COMBO_BOX (menu), display_name);
   }
   gtk_combo_box_set_active (GTK_COMBO_BOX (menu), mapset);
 }
@@ -1141,6 +1138,7 @@ new_game (gboolean shuffle)
 {
   gint i;
   gchar *title;
+  const char *display_name;
 
   /* Reset state */
   game_over = GAME_WAITING;
@@ -1186,7 +1184,9 @@ new_game (gboolean shuffle)
 
   /* Set window title */
   /* Translators: This is the window title for Mahjongg which contains the map name, e.g. 'Mahjongg - Red Dragon' */
-  title = g_strdup_printf (_("Mahjongg - %s"), Q_ (maps[mapset].name));
+
+  display_name = g_dpgettext2 (NULL, "mahjongg map name", maps[mapset].name);
+  title = g_strdup_printf (_("Mahjongg - %s"), display_name);
   gtk_window_set_title (GTK_WINDOW (window), title);
   g_free (title);
   gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
@@ -1358,10 +1358,17 @@ init_scores (void)
 {
   int i;
 
-  highscores = games_scores_new (&scoredesc);
+  highscores = games_scores_new ("mahjongg",
+                                 NULL, 0,
+                                 NULL, NULL,
+                                 0 /* default category; FIXMEchpe: was: "Easy" */,
+                                 GAMES_SCORES_STYLE_TIME_ASCENDING);
 
   for (i = 0; i < nmaps; i++) {
-    games_scores_add_category (highscores, maps[i].score_name, maps[i].name);
+    const char *display_name;
+
+    display_name = g_dpgettext2 (NULL, "mahjongg map name", maps[i].name);
+    games_scores_add_category (highscores, maps[i].score_name, display_name);
   }
 }
 
@@ -1384,22 +1391,11 @@ main (int argc, char *argv[])
   gboolean retval;
   GError *error = NULL;
 
-#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
-  /* If we're going to use gnome-vfs, we need to init threads before
-   * calling any glib functions.
-   */
-  g_thread_init (NULL);
-#endif
-
   if (!games_runtime_init ("mahjongg"))
     return 1;
 
   setgid_io_init ();
 
-  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
-
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
